/* Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above notice and this permission notice shall be included in all copies
 * or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
/* File for "Alpha Blending" lesson of the OpenGL tutorial on
 * www.videotutorialsrock.com
 */



#include <algorithm>
#include <iostream>
#include <math.h>
#include <stdlib.h>
#include <vector>

#ifdef __APPLE__
#include <OpenGL/OpenGL.h>
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include "imageloader.h"
using namespace std;

//For storing the texture Ids of up to 10 textures with OpenGL
GLuint textureId[10];

int indexFace1=0;
int indexFace2=1;

float center1_x = -0.5;
float center1_y = -0.5;
float center2_x = 0.5;
float center2_y = 0.5;

void setupTexture_initRendering() {

	//Set up Texture
	//****************************************************************
	//Just call creatTextureFromBMP and store the texture Id properly
	//****************************************************************
	textureId[0] = creatTextureFromBMP("cloud1.bmp");
	textureId[1] = creatTextureFromBMP("cloud2.bmp");
	textureId[2] = creatTextureFromBMP("grass.bmp");
	textureId[3] = creatTextureFromBMP("stoneWall.bmp");
	textureId[4] = creatTextureFromBMP("trees.bmp");
	textureId[5] = creatTextureFromBMP("fence.bmp");
}



void drawFace1() {

	
	//Use Texture
	//****************************************************************
	//Just enable GL_TEXTURE_2D and provide information through TexCoord
	//****************************************************************	
	
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, textureId[indexFace1]);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	
	glBegin(GL_QUADS);
	
	glTexCoord2f(0, 0);
	glVertex3f(center1_x + -1, center1_y + -1, -5);
	glTexCoord2f(0, 1);
	glVertex3f(center1_x +  1, center1_y + -1, -5);
	glTexCoord2f(1, 1);
	glVertex3f(center1_x +  1, center1_y +  1, -5);
	glTexCoord2f(1, 0);
	glVertex3f(center1_x + -1, center1_y +  1, -5);
	
	glEnd();
	glDisable(GL_TEXTURE_2D);
}

void drawFace2() {

	
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, textureId[indexFace2]);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	
	glBegin(GL_QUADS);
	
	glTexCoord2f(0, 0);
	glVertex3f(center2_x + -1, center2_y + -1, -8);
	glTexCoord2f(0, 1);
	glVertex3f(center2_x +  1, center2_y + -1, -8);
	glTexCoord2f(1, 1);
	glVertex3f(center2_x +  1, center2_y +  1, -8);
	glTexCoord2f(1, 0);
	glVertex3f(center2_x + -1, center2_y +  1, -8);
	
	glEnd();
	glDisable(GL_TEXTURE_2D);
}


void handleKeypress(unsigned char key, int x, int y) {
	switch (key) {
		case 't': //texture change
			indexFace1 = rand() %6;
			indexFace2 = rand() %6;
			while (indexFace1 == indexFace2)
				indexFace2 = rand() %6;

			glutPostRedisplay();

			break;
		case 27: //Escape key
			exit(0);
			break;
	}
}


void drawScene() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	drawFace1();
	drawFace2();

	glutSwapBuffers();
}


void handleResize(int w, int h) {
	glViewport(0, 0, w, h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0, (float)w / (float)h, 1.0, 200.0);
}

int main(int argc, char** argv) {
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(400, 400);
	
	glutCreateWindow("BMP_TextureImageLoader");
	setupTexture_initRendering();
	

	glutDisplayFunc(drawScene);
	glutKeyboardFunc(handleKeypress);
	glutReshapeFunc(handleResize);
	//glutTimerFunc(25, update, 0);
	
	glutMainLoop();
	return 0;
}









